# agfood

> see https://aka.ms/autorest

This is the AutoRest configuration file for agfood.

## Getting Started

To build the SDKs for My API, simply install AutoRest via `npm` (`npm install -g autorest`) and then run:

> `autorest readme.md`

To see additional help and options, run:

> `autorest --help`

For other options on installation see [Installing AutoRest](https://aka.ms/autorest/install) on the AutoRest github page.

---

## Configuration

### Basic Information

These are the global settings for the agfood.

```yaml
openapi-type: arm
tag: package-2020-05-12-preview
```

### Tag: package-2020-05-12-preview

These settings apply only when `--tag=package-2020-05-12-preview` is specified on the command line.

```yaml $(tag) == 'package-2020-05-12-preview'
input-file:
  - Microsoft.AgFoodPlatform/preview/2020-05-12-preview/agfood.json
```

---

# Code Generation

## Swagger to SDK

This section describes what SDK should be generated by the automatic system.
This is not used by Autorest itself.

```yaml $(swagger-to-sdk)
swagger-to-sdk:
  - repo: azure-sdk-for-python-track2
  - repo: azure-sdk-for-go
  - repo: azure-sdk-for-js
```
## Go

See configuration in [readme.go.md](./readme.go.md)

## Python

See configuration in [readme.python.md](./readme.python.md)

## TypeScript

See configuration in [readme.typescript.md](./readme.typescript.md)

## CSharp

See configuration in [readme.csharp.md](./readme.csharp.md)

## Suppression

``` yaml
directive:
  - from: agfood.json
    suppress: R3006 # BodyTopLevelProperties
    where: $.definitions.FarmBeats.properties    
    reason: Currently systemData is not allowed.
  - from: agfood.json
    suppress: R3006 # BodyTopLevelProperties
    where: $.definitions.Extension.properties    
    reason: Currently systemData is not allowed.
  - from: agfood.json
    suppress: R3018  # EnumInsteadOfBoolean
    where:
      - $.definitions.Operation.properties.isDataAction
      - $.definitions.CheckNameAvailabilityResult.properties.nameAvailable 
    reason: Booleans are used to indicate binary states of the property, enum is not appropriate.
  - from: agfood.json
    suppress: R4000  # DescriptionAndTitleMissing
    where:
      - $.definitions.Error.properties.innererror
      - $.definitions.ErrorResponse.properties.error
      - $.definitions.Extension.properties.properties
      - $.definitions.Extension.properties.systemData
      - $.definitions.FarmBeats.properties.properties
      - $.definitions.FarmBeats.properties.systemData
      - $.definitions.InnerError.properties.innererror
      - $.definitions.Operation.properties.display
    reason: Model has description added in it's schema.
  - from: types.json
    suppress: R4000  # DescriptionAndTitleMissing
    where: $.definitions.Resource
    reason: Common type models are inherited.
```
