# AppConfiguration

> see https://aka.ms/autorest

This is the AutoRest configuration file for AppConfiguration.

---

## Getting Started

To build the SDK for AppConfiguration, simply [Install AutoRest](https://aka.ms/autorest/install) and in this folder, run:

> `autorest`

To see additional help and options, run:

> `autorest --help`

---

## Configuration

### Basic Information

These are the global settings for the AppConfiguration API.

``` yaml
openapi-type: arm
tag: package-2021-03-01-preview
```

### Tag: package-2021-03-01-preview

These settings apply only when `--tag=2021-03-01-preview` is specified on the command line.

``` yaml $(tag) == 'package-2021-03-01-preview'
input-file:
- Microsoft.AppConfiguration/preview/2021-03-01-preview/appconfiguration.json
```

### Tag: package-2020-07-01-preview

These settings apply only when `--tag=2020-07-01-preview` is specified on the command line.

``` yaml $(tag) == 'package-2020-07-01-preview'
input-file:
- Microsoft.AppConfiguration/preview/2020-07-01-preview/appconfiguration.json
```

### Tag: package-2020-06-01

These settings apply only when `--tag=package-2020-06-01` is specified on the command line.

``` yaml $(tag) == 'package-2020-06-01'
input-file:
- Microsoft.AppConfiguration/stable/2020-06-01/appconfiguration.json
```

### Tag: package-2019-11-01-preview

These settings apply only when `--tag=package-2019-11-01-preview` is specified on the command line.

``` yaml $(tag) == 'package-2019-11-01-preview'
input-file:
- Microsoft.AppConfiguration/preview/2019-11-01-preview/appconfiguration.json
```

### Tag: package-2019-02-01-preview

These settings apply only when `--tag=package-2019-02-01-preview` is specified on the command line.

``` yaml $(tag) == 'package-2019-02-01-preview'
input-file:
- Microsoft.AppConfiguration/preview/2019-02-01-preview/appconfiguration.json
```

### Tag: package-2019-10-01

These settings apply only when `--tag=package-2019-10-01` is specified on the command line.

``` yaml $(tag) == 'package-2019-10-01'
input-file:
- Microsoft.AppConfiguration/stable/2019-10-01/appconfiguration.json
```

---

# Code Generation

## Swagger to SDK

This section describes what SDK should be generated by the automatic system.
This is not used by Autorest itself.

``` yaml $(swagger-to-sdk)
swagger-to-sdk:
  - repo: azure-powershell
  - repo: azure-sdk-for-python
  - repo: azure-sdk-for-python-track2
  - repo: azure-sdk-for-java
  - repo: azure-sdk-for-net
  - repo: azure-sdk-for-js
  - repo: azure-sdk-for-go
  - repo: azure-sdk-for-ruby
    after_scripts:
      - bundle install && rake arm:regen_all_profiles['azure_mgmt_app_configuration']
  - repo: azure-resource-manager-schemas
```

## C#

See configuration in [readme.csharp.md](./readme.csharp.md)

## Python

See configuration in [readme.python.md](./readme.python.md)

## Java

See configuration in [readme.java.md](./readme.java.md)

## Go

See configuration in [readme.go.md](./readme.go.md)

## Ruby

See configuration in [readme.ruby.md](./readme.ruby.md)

## CLI

See configuration in [readme.cli.md](./readme.cli.md)

## Suppression

``` yaml
directive:
  - suppress: EnumInsteadOfBoolean
    from: appconfiguration.json
    where: $.definitions.NameAvailabilityStatus.properties.nameAvailable
    reason: This is standard for check name availability.
  - suppress: EnumInsteadOfBoolean
    from: appconfiguration.json
    where: $.definitions.ApiKey.properties.readOnly
    reason: We did consider using an enum instead but found it to not be helpful.
  - suppress: EnumInsteadOfBoolean
    from: appconfiguration.json
    where: $.definitions.KeyValueProperties.properties.locked
    reason: This is data plane level information proxied through the RP and cannot be changed.
  - suppress: EnumInsteadOfBoolean
    from: appconfiguration.json
    where: $.definitions.ConfigurationStoreProperties.properties.disableLocalAuth
    reason: This is a standardized ARM API.
  - suppress: EnumInsteadOfBoolean
    from: appconfiguration.json
    where: $.definitions.ConfigurationStorePropertiesUpdateParameters.properties.disableLocalAuth
    reason: This is a standardized ARM API.
  - suppress: EnumInsteadOfBoolean
    from: appconfiguration.json
    where: $.definitions.OperationDefinition.properties.isDataAction
    reason: This is a standardized ARM API.
```



