# Attestation
    
> see https://aka.ms/autorest

This is the AutoRest configuration file for Attestation.



---
## Getting Started 
To build the SDK for Attestation, simply [Install AutoRest](https://aka.ms/autorest/install) and in this folder, run:

> `autorest`

To see additional help and options, run:

> `autorest --help`
---

## Configuration
Following are the settings for using this specification with [AutoRest](https://aka.ms/autorest) tool to validation and optionally generate SDK.

### Basic Information
These are the global settings for the Attestation APIs.

``` yaml
openapi-type: data-plane
tag: package-2020-10-01
```

### Tag: package-2018-09-01

These settings apply only when `--tag=package-2018-09-01` is specified on the command line.

``` yaml $(tag) == 'package-2018-09-01'
input-file:
- Microsoft.Attestation/stable/2018-09-01-preview/attestation.json
```

### Tag: package-2020-10-01

These settings apply only when `--tag=package-2020-10-01` is specified on the command line.

``` yaml $(tag) == 'package-2020-10-01'
input-file:
- Microsoft.Attestation/stable/2020-10-01/attestation.json
```

---
# Code Generation

## Swagger to SDK

This section describes what SDK should be generated by the automatic system.
This is not used by Autorest itself.

``` yaml $(swagger-to-sdk)
swagger-to-sdk:
  - repo: azure-sdk-for-net
  - repo: azure-sdk-for-python
  - repo: azure-sdk-for-go
```

## Python

See configuration in [readme.python.md](./readme.python.md)

---
## C# 

These settings apply only when `--csharp` is specified on the command line.
Please also specify `--csharp-sdks-folder=<path to "SDKs" directory of your azure-sdk-for-net clone>`.

``` yaml $(csharp)
csharp:
  azure-arm: true
  license-header: MICROSOFT_MIT_NO_VERSION
  namespace: Microsoft.Azure.Attestation
  output-folder: $(csharp-sdks-folder)/attestation/Microsoft.Azure.Attestation/src/Generated
  clear-output-folder: true
```

## Go

See configuration in [readme.go.md](./readme.go.md)

## Multi-API/Profile support for AutoRest v3 generators 

AutoRest V3 generators require the use of `--tag=all-api-versions` to select api files.

This block is updated by an automatic script. Edits may be lost!

``` yaml $(tag) == 'all-api-versions' /* autogenerated */
# include the azure profile definitions from the standard location
require: $(this-folder)/../../../profiles/readme.md

# all the input files across all versions
input-file:
  - $(this-folder)/Microsoft.Attestation/stable/2018-09-01-preview/attestation.json
  - $(this-folder)/Microsoft.Attestation/stable/2020-10-01/attestation.json
```

If there are files that should not be in the `all-api-versions` set, 
uncomment the  `exclude-file` section below and add the file paths.

``` yaml $(tag) == 'all-api-versions'
#exclude-file: 
#  - $(this-folder)/Microsoft.Example/stable/2010-01-01/somefile.json
```

### Suppression
``` yaml
directive:
  - suppress: DefinitionsPropertiesNamesCamelCase
    from: attestation.json
    where: $.definitions.StoredAttestationPolicy.properties.AttestationPolicy
    reason: Existing Clients use these definitions which must be maintained.
  - suppress: DefinitionsPropertiesNamesCamelCase
    from: attestation.json
    where: $.definitions.AttestationResult.properties.policy_signer
    reason: Existing Clients use these definitions which must be maintained.
  - suppress: DefinitionsPropertiesNamesCamelCase
    from: attestation.json
    where: $.definitions.AttestationResult.properties.policy_hash
    reason: Existing Clients use these definitions which must be maintained.
  - suppress: DefinitionsPropertiesNamesCamelCase
    from: attestation.json
    where: $.definitions.AttestationResult.properties.rp_data
    reason: Existing Clients use these definitions which must be maintained.
```

