# MariaDB

> see https://aka.ms/autorest

This is the AutoRest configuration file for MariaDB.



---
## Getting Started
To build the SDK for MariaDB, simply [Install AutoRest](https://aka.ms/autorest/install) and in this folder, run:

> `autorest`

To see additional help and options, run:

> `autorest --help`
---

## Configuration



### Basic Information
These are the global settings for the MariaDB API.

``` yaml
title: MariaDBManagementClient
description: The Microsoft Azure management API provides create, read, update, and delete functionality for Azure MariaDB resources including servers, databases, firewall rules, VNET rules, log files and configurations with new business model.
openapi-type: arm
tag: package-2020-01-01
```

### Tag: package-2018-06-01-preview

These settings apply only when `--tag=package-2018-06-01-preview` is specified on the command line.

``` yaml $(tag) == 'package-2018-06-01-preview'
input-file:
- Microsoft.DBforMariaDB/preview/2018-06-01-preview/mariadb.json
```


### Tag: package-2018-06-01

These settings apply only when `--tag=package-2018-06-01` is specified on the command line.

``` yaml $(tag) == 'package-2018-06-01'
input-file:
- Microsoft.DBforMariaDB/stable/2018-06-01/mariadb.json
- Microsoft.DBforMariaDB/stable/2018-06-01/QueryPerformanceInsights.json
- Microsoft.DBforMariaDB/stable/2018-06-01/PerformanceRecommendations.json
- Microsoft.DBforMariaDB/stable/2018-06-01/PrivateEndpointConnections.json
- Microsoft.DBforMariaDB/stable/2018-06-01/PrivateLinkResources.json
- Microsoft.DBforMariaDB/stable/2018-06-01/ServerSecurityAlertPolicies.json
```


### Tag: package-2018-06-01-privatepreview

These settings apply only when `--tag=package-2018-06-01-privatepreview` is specified on the command line.

``` yaml $(tag) == 'package-2018-06-01-privatepreview'
input-file:
- Microsoft.DBforMariaDB/preview/2018-06-01-privatepreview/mariadb.json
- Microsoft.DBforMariaDB/preview/2018-06-01-privatepreview/PrivateEndpointConnections.json
- Microsoft.DBforMariaDB/preview/2018-06-01-privatepreview/PrivateLinkResources.json
```

### Tag: package-2020-01-01-privatepreview

These settings apply only when `--tag=package-2020-01-01-privatepreview` is specified on the command line.


``` yaml $(tag) == 'package-2020-01-01-privatepreview'
input-file:
- Microsoft.DBforMariaDB/preview/2020-01-01-privatepreview/DataEncryptionKeys.json
```


### Tag: package-2020-01-01

These settings apply only when `--tag=package-2020-01-01` is specified on the command line.

``` yaml $(tag) == 'package-2020-01-01'
input-file:
- Microsoft.DBforMariaDB/stable/2018-06-01/mariadb.json
- Microsoft.DBforMariaDB/stable/2018-06-01/QueryPerformanceInsights.json
- Microsoft.DBforMariaDB/stable/2018-06-01/PerformanceRecommendations.json
- Microsoft.DBforMariaDB/stable/2018-06-01/PrivateEndpointConnections.json
- Microsoft.DBforMariaDB/stable/2018-06-01/PrivateLinkResources.json
- Microsoft.DBforMariaDB/stable/2018-06-01/ServerSecurityAlertPolicies.json
- Microsoft.DBforMariaDB/stable/2020-01-01/Servers.json
```

## Suppression
``` yaml
directive:
  - suppress: PathResourceProviderNamePascalCase
    reason: The name of the provider is Microsoft.DBforMariaDB
```

---
# Code Generation


## Swagger to SDK

This section describes what SDK should be generated by the automatic system.
This is not used by Autorest itself.

``` yaml $(swagger-to-sdk)
swagger-to-sdk:
  - repo: azure-sdk-for-net
  - repo: azure-sdk-for-python
  - repo: azure-sdk-for-java
  - repo: azure-sdk-for-go
  - repo: azure-sdk-for-js
  - repo: azure-sdk-for-node
  - repo: azure-resource-manager-schemas
```


### C#

These settings apply only when `--csharp` is specified on the command line.
Please also specify `--csharp-sdks-folder=<path to "SDKs" directory of your azure-sdk-for-net clone>`.

``` yaml $(csharp)
csharp:
  azure-arm: true
  license-header: MICROSOFT_MIT_NO_VERSION
  namespace: Microsoft.Azure.Management.MariaDB
  output-folder: $(csharp-sdks-folder)/mariadb/Microsoft.Azure.Management.MariaDB/src/Generated
  clear-output-folder: true
```


## Python

These settings apply only when `--python` is specified on the command line.
Please also specify `--python-sdks-folder=<path to the root directory of your azure-sdk-for-python clone>`.
Use `--python-mode=update` if you already have a setup.py and just want to update the code itself.

``` yaml $(python) && !$(track2)
python-mode: create
python:
  azure-arm: true
  license-header: MICROSOFT_MIT_NO_VERSION
  payload-flattening-threshold: 2
  namespace: azure.mgmt.rdbms.mariadb
  package-name: azure-mgmt-rdbms
  clear-output-folder: true
```
``` yaml $(python) && $(track2)
python-mode: create
azure-arm: true
license-header: MICROSOFT_MIT_NO_VERSION
namespace: azure.mgmt.rdbms.mariadb
package-name: azure-mgmt-rdbms
clear-output-folder: true
modelerfour:
  lenient-model-deduplication: true
```
``` yaml $(python) && $(python-mode) == 'update' && !$(track2)
python:
  no-namespace-folders: true
  output-folder: $(python-sdks-folder)/rdbms/azure-mgmt-rdbms/azure/mgmt/rdbms/mariadb
```
``` yaml $(python) && $(python-mode) == 'create' && !$(track2)
python:
  basic-setup-py: true
  output-folder: $(python-sdks-folder)/rdbms/azure-mgmt-rdbms
```
``` yaml $(python) && $(python-mode) == 'update' && $(track2)
no-namespace-folders: true
output-folder: $(python-sdks-folder)/rdbms/azure-mgmt-rdbms/azure/mgmt/rdbms/mariadb
```
``` yaml $(python) && $(python-mode) == 'create' && $(track2)
basic-setup-py: true
output-folder: $(python-sdks-folder)/rdbms/azure-mgmt-rdbms
```

## Go

See configuration in [readme.go.md](./readme.go.md)

## Java

See configuration in [readme.java.md](./readme.java.md)



